// Search local filesystem for java class files.
// 11 Jun 1997 Derek Lieber
//
package jd;
import java.util.*;
import java.io.*;
import java.util.zip.*;

class ClassFinder 
   implements Constants
   {
   private Vector repositories;
   
   ClassFinder(Vector classRepositories)
      {
      this.repositories = classRepositories;
      }

   // Discover package to which specified class belongs.
   // Taken:    class name   - something like "java.lang.Object"
   // Returned: package name - something like "java.lang"
   //
   static String
   packageForClass(String className)
      {
      int dotIndex = className.lastIndexOf('.');
      return (dotIndex == -1) ? "" : className.substring(0, dotIndex);
      }

   // Discover names of classes that were compiled from specified source file.
   // Taken:    source file name - something like "d:\foo\bar\Main.java"
   // Returned: class names      - something like { "Main", "Main$Inner", "Main$1" }
   //
   synchronized
   Vector
   classesForSource(String sourceFileName)
      {
      String packageName = "";
      
      // first discover outer class names by parsing source file
      //
      Vector outerClassNames = new Vector();
      try
         {
         BufferedReader  reader    = new BufferedReader(new InputStreamReader(new FileInputStream(sourceFileName)));
         StreamTokenizer tokenizer = new StreamTokenizer(reader);
         int             nesting   = 0;

         tokenizer.slashStarComments(true);
         tokenizer.slashSlashComments(true);

         for (;;)
            {
            int ttype = tokenizer.nextToken();
            if (ttype == tokenizer.TT_EOF)
               break;
            if (ttype == '{')
               {
               ++nesting;
               continue;
               }
            if (ttype == '}')
               {
               --nesting;
               continue;
               }
            if (nesting != 0)
               continue;
            if (ttype != tokenizer.TT_WORD)
               continue;
            if (tokenizer.sval.equals("package"))
               { // assumption: package statement is well formed
               loop: for (;;)
                  switch (tokenizer.nextToken())
                     {
                     case tokenizer.TT_WORD: packageName += tokenizer.sval; continue;
                     case '_':               packageName += '_';            continue;
                     case '.':               packageName += '.';            continue;
                     case ';':                                              break loop;
                     }
               continue;
               }
            if (tokenizer.sval.equals("class"))
               { // assumption: outer class statement is well formed
               String className = "";
               loop: for (;;)
                  switch (tokenizer.nextToken())
                     {
                     case tokenizer.TT_WORD: if (tokenizer.sval.equals("extends") || 
                                                 tokenizer.sval.equals("implements")) 
                                                 break loop;
                                             className += tokenizer.sval; continue;
                     case '_':               className += '_';            continue;
                     case '.':               className += '.';            continue;
                     case '{':                                            break loop;
                     }
               tokenizer.pushBack(); // 'extends', 'implements', or '{'
               outerClassNames.addElement(className);
               }
            }

         reader.close();
         }

      catch (IOException x)
         {
         Session.popup("classFinder: " + x);
         }
      
      // now discover inner classes by searching class repositories for .class files that have same prefix as outer class names
      // example:    Main             <-- outer class name
      //             Main$Inner.class <-- inner class file
      //             Main$1.class     <-- anonymous class file
      //
      Vector results = new Vector();

      String packagePrefix = packageName.length() == 0 ? "" : packageName + ".";            // aaa.
      for (Enumeration o = outerClassNames.elements(); o.hasMoreElements(); )
         {
         String outerClassName = (String)o.nextElement();                                   // bbb

         results.addElement(packagePrefix + outerClassName);

         for (Enumeration r = repositories.elements(); r.hasMoreElements(); )
            {
            String  repositoryName = (String)r.nextElement();
            boolean foundInnerClasses = false;

            try
               {
               if (repositoryName.endsWith(".zip") || repositoryName.endsWith(".jar"))      // d:\xxx.zip
                  {
                  ZipFile archive = new ZipFile(repositoryName);
                  String  prefix  = packagePrefix.replace('.', '/') + outerClassName + "$"; // aaa/bbb$

                  for (Enumeration a = archive.entries(); a.hasMoreElements(); )
                     {
                     String entryName = ((ZipEntry)a.nextElement()).getName();              // aaa/bbb$123.class
                     int    tailIndex = entryName.lastIndexOf(".class");

                     if (tailIndex < 0)
                        continue;

                     String innerClassName = entryName.substring(0, tailIndex);             // aaa/bbb$123
                     if (innerClassName.startsWith(prefix))
                        {
                        results.addElement(innerClassName.replace('/', '.'));               // aaa.bbb$123
                        foundInnerClasses = true;
                        }
                     }

                  archive.close();
                  }
               else
                  {
                  String dirName = repositoryName                                           // d:\xxx
                                 + FILE_PATH_SEPARATOR
                                 + packagePrefix.replace('.', FILE_PATH_SEPARATOR);         // d:\xxx\aaa\
                  String  prefix = outerClassName + "$";                                    // bbb$

                  File dir = new File(dirName);
                  if (dir.isDirectory() == false)
                     continue;

                  String files[] = dir.list();
                  if (files == null)
                     continue;

                  for (int i = 0, n = files.length; i < n; ++i)
                     {
                     String entryName = files[i];
                     int    tailIndex = entryName.lastIndexOf(".class");

                     if (tailIndex < 0)
                        continue;

                     String innerClassName = entryName.substring(0, tailIndex);             // bbb$123
                     if (innerClassName.startsWith(prefix))
                        {
                        results.addElement(packagePrefix + innerClassName);                 // aaa.bbb$123
                        foundInnerClasses = true;
                        }
                     }
                  }
               }


            catch(IOException x)
               {
               Session.popup("classFinder: " + x);
               }

            if (foundInnerClasses)
               break;
            }
         }

      return results;
      }
   }
