// Edit a path specification string within a modal popup window.
// See also: MessageDialog
// 30 Jul 1997 Derek Lieber
//
package jd;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import ui.MultiButton;

class PathDialog extends Dialog
   implements Constants
   {
   private Frame     frame;
   private TextArea  textArea;
   private Container buttonPanel;
   private char      pathSeparator;
   private String    result;

   // Create popup editor.
   // Taken: frame relative to which we should pop up
   //        string to display in dialog's title bar
   //        path string to be edited - something like ".;\foo\bar;\java\classes.zip"
   //        path separator character - something like ';'
   //
   PathDialog(Frame _frame, String title, String pathString, char _pathSeparator)
      {
      super(_frame, title, true);

      frame         = _frame;
      textArea      =  new TextArea();
      buttonPanel   =  new Panel();
      pathSeparator = _pathSeparator;
      result        =  null;

      setLayout(new BorderLayout());
      add("Center", textArea);
      add("South",  buttonPanel);
      
      MultiButton okButton     = new MultiButton(" Ok ", null);
      MultiButton cancelButton = new MultiButton(" Cancel ", null);
      
      okButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); }});
      cancelButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); }});
      addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { cancel(); }});

      buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
      buttonPanel.add("", okButton);
      buttonPanel.add("", cancelButton);

      // explode path specification string into editing buffer
      //
      StringBuffer buf = new StringBuffer();
      for (StringTokenizer st = new StringTokenizer(pathString, String.valueOf(pathSeparator), false); st.hasMoreTokens(); )
         {
         buf.append(st.nextToken());
         buf.append('\n');
         }
      textArea.append(buf.toString());

      textArea.setEditable(true);
      textArea.setFont(MONOSPACED_FONT);
      }

   // Allow path to be edited.
   // Taken:    nothing
   // Returned: revised path (null --> "cancel" was hit)
   // Note: to avoid a deadlock between Dialog.show() and Dialog.ok(), this method must NOT be synchronized!
   //
// synchronized
   String
   edit()
      {
      pack();
      Point     loc = frame.getLocation();
      Dimension dim = frame.getSize();
      setBounds(loc.x + 20, loc.y + 20, dim.width / 3, dim.height / 3);
      show(); // note: dialog is "modal", meaning that control blocks here until ok() or cancel() hides the window again
      dispose();
      return result;
      }

   synchronized
   void
   ok()
      {
      result = "";
      
      // convert editing buffer back to path specification string
      //
      String text = textArea.getText() + "\n";
      for (int beg = 0, end = 0, n = text.length(); beg < n; ++end)
         {
         if (text.charAt(end) != '\n')
            continue;

         if (beg != end)
            {
            if (result.length() != 0)
               result += pathSeparator;
            result += text.substring(beg, end);
            }

         beg = end + 1;
         }
      
      // correct any misuses of slash or backslash
      //
      if (FILE_PATH_SEPARATOR == '\\')
         result = result.replace('/', '\\');
      else
         result = result.replace('\\', '/');

      // AWT bug workaround: prevent desktop from getting focus when dialog vanishes
      //
      frame.toFront();      
      Thread.currentThread().yield();
      
      // unblock caller at site of "dialog.show()"
      //
      setVisible(false);
      }

   synchronized
   void
   cancel()
      {
      result = null;
      
      // AWT bug workaround: prevent desktop from getting focus when dialog vanishes
      //
      frame.toFront();      
      Thread.currentThread().yield();
      
      // unblock caller at site of "dialog.show()"
      //
      setVisible(false);
      }
   }
