// Panel with title, text area, and scrollbars.
// 01 Mar 1996 Derek Lieber
//
package jd;
import  java.awt.*;
import  java.awt.event.*;
import  ui.*;
import  util.Tracer;

class TextPanel extends FastPanel
   implements Constants, ui.TextListener, ActionListener
   {
   private Debugger     debugger;         // place to get latest snapshot of data to be displayed in this panel
   private Container    titlebar;         // titlebar
   private MultiLabel   label;            // titlebar string
   private TextScroller scroller;         // text display area
   private Snapshot     previousSnapshot; // snapshot previously displayed in this panel
   private Snapshot     currentSnapshot;  // snapshot currently being displayed in this panel
   
   //-----------//
   // interface //
   //-----------//

   // Create.
   //
   TextPanel(String name, String title, Debugger debugger)
      {
      this.debugger = debugger;
      this.titlebar = new FastPanel();
      this.label    = new MultiLabel(title);
      this.scroller = new TextScroller();
      
      setName(name + "Panel");
      scroller.setName(name);
      scroller.setFont(MONOSPACED_FONT);
      
      titlebar.setLayout(new BorderLayout());
      titlebar.add("Center", label);

      setLayout(null);
      add(titlebar);
      add(scroller);
      
      // initial decorations: none
      //
      scroller.setScrollbarAvailability(false);

      // adjust decorations as mouse enters our borders
      //
      addMouseListener(new MouseAdapter() 
         { 
         public void mouseEntered(MouseEvent e) { TextPanel.mouseEnteredOrMovedOn(TextPanel.this); }
         });

      titlebar.addMouseListener(new MouseAdapter() 
         { 
         public void mouseEntered(MouseEvent e) { TextPanel.mouseEnteredOrMovedOn(TextPanel.this); }
         public void mouseClicked(MouseEvent  e) { Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.ZOOM, TextPanel.this)); }
         });

      scroller.canvas.addMouseMotionListener(new MouseMotionAdapter() 
         { 
         public void mouseMoved(MouseEvent e) { TextPanel.mouseEnteredOrMovedOn(TextPanel.this); } 
         });

      // capture text area clicks
      //
      scroller.addTextListener(this);
      
   // setCursor(scroller.getCursor());
   // titlebar.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      }

   // The data associated with this panel may have changed.
   // If so, update the display to match.
   // Runs in "Debugger" thread.
   //
   synchronized
   void
   synchronizeView()
      {
   // Tracer.trace("synchronizeView", getName());
   
      //!!TODO: only do this if we're currently visible on screen?

      // fetch new data
      //
      previousSnapshot = currentSnapshot;
      currentSnapshot  = debugger.getSnapshotFor(this);
      
      // update text to match new data
      //
      if (currentSnapshot != null)
         currentSnapshot.update(scroller, previousSnapshot);

      // update screen with visible portion of text
      //
      if (scroller.needsRedraw())
         Util.redraw(scroller);
      }

   // Bring specified string into view.
   //
   void
   search(String searchString, int direction)
      {
      int lino = scroller.search(searchString, direction);
      if (lino == -1)
         return;
      scroller.setViewpoint(lino);
      scroller.setMarkedLine(lino);
      scroller.setSearchLine(lino);
      if (scroller.needsRedraw())
         Util.redraw(scroller);
      }
   
   // Bring specified line (0-indexed) into view.
   //
   void
   gotoLine(int lino)
      {
      scroller.setViewpoint(lino);
      scroller.setMarkedLine(lino);
      scroller.setSearchLine(lino);
      if (scroller.needsRedraw())
         Util.redraw(scroller);
      }
   
   //------------------------//
   // implementation details //
   //------------------------//
   
   // Adjust layout to accomodate size change.
   // Runs in AWT thread.
   //
   synchronized
   public void
   doLayout() //- overrides Container
      {
      int x = 0;
      int y = 0;
      int w = getSize().width;
      int h = getSize().height;
      
      Dimension titlebarSize  = titlebar.getPreferredSize();
      int       titlebarW     = w;
      int       titlebarH     = titlebarSize.height;
   
      titlebar.setBounds(x, y, titlebarW, titlebarH);
      scroller.setBounds(x, y + titlebarH, w, h - titlebarH);
      }

   // Respond to (left) mouse clicks on text.
   // Runs in "AWT" thread.
   //
   public void
   textClicked(String text, int lino, int colno) //- implements TextListener
      {
      scroller.setMarkedLine(lino);
      scroller.setSearchLine(lino);
      if (scroller.needsRedraw())
         Util.redraw(scroller);
      
      Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.PICK, currentSnapshot, text, lino, colno));
      }

   // Respond to (right) mouse clicks on text.
   // Runs in "AWT" thread.
   //
   public void
   textMetaClicked(Component c, int x, int y) //- implements TextListener
      {
      PopupMenu menu = new PopupMenu();
      MenuItem  menuItem;
      add(menu);
      menu.add(menuItem = new MenuItem(SEARCHB_TITLE)); menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem(SEARCHF_TITLE)); menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem(GOTO_TITLE));    menuItem.addActionListener(this);
      menu.show(c, x, y);
      Thread.currentThread().yield(); // otherwise menu action listeners never get called (jdk 1.1.4)
      remove(menu);
      }

   // Respond to popup menu hits.
   //
   synchronized
   public void
   actionPerformed(ActionEvent e) //- implements ActionListener
      {
      String cmd = e.getActionCommand();
      if (cmd.equals(SEARCHB_TITLE)) Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.SEARCHB, this));
      if (cmd.equals(SEARCHF_TITLE)) Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.SEARCHF, this));
      if (cmd.equals(GOTO_TITLE))    Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.GOTO,    this));
      }

   // Add/remove decorations as mouse traverses a text panel.
   // We fix up border crossing information that gets lost by AWT 1.1.4 when
   // the mouse is moved too quickly by using mouse "move" reports
   // to augment mouse "enter" reports. Sigh.
   //
   static Component currentDecorationTarget;

   synchronized
   static void
   mouseEnteredOrMovedOn(Component target)
      {
      if (currentDecorationTarget == target)
         return;
         
      if (currentDecorationTarget != null && currentDecorationTarget instanceof TextPanel)
         { // remove decorations from panel that used to contain mouse
         TextPanel p = (TextPanel)currentDecorationTarget;
         p.scroller.setScrollbarAvailability(false);
         if (p.scroller.needsRedraw())
            Util.redraw(p.scroller);
         }

      if (target instanceof TextPanel)
         { // apply decorations to panel that now contains mouse
         currentDecorationTarget = target;
         
         TextPanel p = (TextPanel)currentDecorationTarget;
         p.scroller.setScrollbarAvailability(true);
         if (p.scroller.needsRedraw())
            Util.redraw(p.scroller);
         
         return;
         }

      currentDecorationTarget = null;
      }
   }
