// An incrementally updatable tree for object inspection.
// See also: ClassTree
// 01 Jul 1997 Derek Lieber
//
//
package jd;
import  sun.tools.debug.*;
import  java.util.Vector;
import  java.util.Enumeration;

class ValueTree
   {
   String      indentation; // indentation level for this subtree
   int         lino;        // line number for this subtree
   
   RemoteClass qualifier;   // if value is a "field", here's the class from which it was inherited (null --> not a field)
   String      identifier;  // identifier of local variable, field, or array element whose value is displayed in this subtree (null --> nothing to display)
   RemoteValue value;       // value to be displayed
   
   Vector      components;  // if value is a "composite", here are its fields or array elements (null --> no details visible)
   String      text;        // printable representation of value
   
   final static int MAXELTS = 10; // maximum number of array elements to display
   final static int MAXSTR  = 50; // maximum number of string characters to display

   ValueTree(RemoteClass qualifier, String identifier, RemoteValue value, String indentation)
      throws Exception
      {
      this.indentation = indentation;
      this.lino        = 0;
      
      this.qualifier   = qualifier;
      this.identifier  = identifier;
      this.value       = value;
      this.components  = null;
      
      StringBuffer buf = new StringBuffer(); 
      if (identifier != null)
         printValue(buf, qualifier, identifier, value);
      this.text        = buf.toString();
      }

   // Expand or collapse subtree at specified line number.
   // Taken:    line number sought
   // Returned: was subtree found?
   //
   synchronized
   boolean
   expandOrCollapse(int targetLino)
      throws Exception
      {
      if (lino == targetLino)
         {
         if (components == null)
            { // expand detail
            components = new Vector();
            populate();
            }
         else
            { // collapse detail
            components = null;
            }
         return true;
         }
         
      if (components != null)
         for (Enumeration e = components.elements(); e.hasMoreElements(); )
            {
            ValueTree tree = (ValueTree)e.nextElement();
            if (tree.expandOrCollapse(targetLino))
               return true;
            }

      return false;
      }

   // If this node corresponds to a composite object, then populate "components" with the object's fields or array elements.
   //
   synchronized
   void
   populate()
      throws Exception
      {
      // null value?
      //
      if (value == null)
         return;

      // primitive value?
      //
      if (!value.isObject() && !value.typeName().equals("array"))
         return;

      RemoteObject object = (RemoteObject)value;
      RemoteClass cls = object.getClazz();
      if (cls == null) { return; } // bug in jdk 1.1.4: getClazz returns null on java.lang.Object
      String className = cls.getName();

      // array value?
      //
      if (className.startsWith("["))
         {
         RemoteArray ary = (RemoteArray)object;
         int         len = ary.getSize();
         int         max = len > MAXELTS ? MAXELTS : len;
         if (max != 0)
            {
            RemoteValue[] elts = ary.getElements(0, max - 1);

            //!!TODO: someday allow (multi-dimensional) array slices to be selected. for now just print a few elements

            for (int i = 0; i < max; ++i)
               {
               RemoteClass elementQualifier  = null;
               String      elementIdentifier = "[" + i + "]";
               RemoteValue elementValue      = elts[i];

               components.addElement(new ValueTree(elementQualifier, elementIdentifier, elementValue, indentation + "   "));
               }
            }

         return;
         }

      // object value
      //
      RemoteField fields[] = object.getFields();
      for (int i = 0, n = fields.length; i < n; ++i)
         {
         RemoteField field           = fields[i];
         RemoteClass fieldQualifier  = findQualifier(cls, i);
         String      fieldIdentifier = field.getName();
         RemoteValue fieldValue      = object.getFieldValue(fieldIdentifier);

         components.addElement(new ValueTree(fieldQualifier, fieldIdentifier, fieldValue, indentation + "   "));
         }
      }

   // Find superclass from which specified field was inherited.
   // Taken: class containing field
   //        field's slot number
   //
   static
   private RemoteClass
   findQualifier(RemoteClass cls, int slot)
      throws Exception
      {
      RemoteClass superclass           = cls.getSuperclass();
      int         superclassFieldCount = superclass.getInstanceFields().length;
      return (slot < superclassFieldCount) ? findQualifier(superclass, slot) : cls;
      }

   // Renumber tree components after expanding/collapsing a subtree.
   //
   synchronized
   int
   renumber(int counter)
      {
      lino = counter++;
      
      if (components != null)
         for (Enumeration e = components.elements(); e.hasMoreElements(); )
            {
            ValueTree tree = (ValueTree)e.nextElement();
            counter = tree.renumber(counter);
            }
            
      return counter;
      }

   // Generate (possibly multi-line) textual description of "this" RemoteValue.
   //
   synchronized
   void
   print(StringBuffer out)
      throws Exception
      {
      out.append(indentation);
      out.append(text);
      
      if (components == null)
         return;
         
      for (Enumeration e = components.elements(); e.hasMoreElements(); )
         {
         ValueTree tree = (ValueTree)e.nextElement();
         tree.print(out);
         }
      }
   
   // Generate (one-line) textual description of specified RemoteValue.
   // We try to print something compact and useful.
   // We also try to avoid transferring large amounts of data (eg. buffers and arrays) across the debugging socket.
   // We also try to avoid printing non-ascii text (eg. newlines and other control characters).
   //
   static void
   printValue(StringBuffer out, RemoteClass qualifier, String identifier, RemoteValue value)
      throws Exception
      {
      String prefix = (qualifier == null) ? "" : qualifier.typeName() + ".";
      
      out.append(prefix);
      out.append(identifier);
      out.append(": ");

      // null object?
      //
      if (value == null)
         {
         out.append("null\n");
         return;
         }

      // primitive?
      //
      if (!value.isObject() && !value.typeName().equals("array"))
         {
         if (value.typeName().equals("char")) out.append(formatCharValue(value));
         else                                 out.append(value.toString());
         out.append("\n");
         return;
         }
      
      if (prefix.equals("java.lang.Throwable.") && identifier.equals("backtrace"))
         { // careful: Sun's jvm stores non-java data in this field, and
           // attempting to print it would cause a runtime cast exception
         out.append("<internal data>\n");
         return;
         }

      RemoteClass cls = ((RemoteObject)value).getClazz();
      if (cls == null) { out.append("Class(java.lang.Object)\n"); return; } // bug in jdk 1.1.4: getClazz returns null on java.lang.Object
      String className = cls.getName();

      // array object?
      //
      if (className.startsWith("["))
         {
         int dim = 0, dimIndex = 0;
         for (; className.charAt(dimIndex) == '['; ++dimIndex)
            dim += 1;

         char   elementType = className.charAt(dimIndex);
         String signature   = "?";
         switch (elementType)
            {
            case 'Z': signature = "boolean"; break;
            case 'C': signature = "char";    break;
            case 'F': signature = "float";   break;
            case 'D': signature = "double";  break;
            case 'B': signature = "byte";    break;
            case 'S': signature = "short";   break;
            case 'I': signature = "int";     break;
            case 'J': signature = "long";    break;
            case 'L': signature = className.substring(dimIndex + 1, className.length() - 1); break;
            }

         for (int i = 0; i < dim; ++i)
            signature += "[]"; //!!TODO: embed each array dimension inside the "[]" by querying remote value's "array.length"

         if (signature.startsWith("java.lang."))
            signature = signature.substring("java.lang.".length());

         out.append(signature);
         out.append(" = { ");

         RemoteArray ary = (RemoteArray)value;
         int         len = ary.getSize();
         int         max = len > MAXELTS ? MAXELTS : len;
         if (max != 0)
            {
            RemoteValue[] elts = ary.getElements(0, max - 1);

            for (int i = 0; i < max; )
               {
               RemoteValue elt = elts[i];
               if (elt == null)
                  out.append("null");
               else
                  switch (elementType)
                     {
                     case 'Z':
                     case 'F':
                     case 'D':
                     case 'B':
                     case 'S':
                     case 'I':
                     case 'J': out.append(elt.toString()); break;
                     case 'C': out.append(formatCharValue(elt)); break;
                     case 'L': out.append(signature.substring(0, signature.length() - dimIndex * 2)); break;
                     }

               if (++i < max)
                  {
                  out.append(", ");
                  continue;
                  }

               if (len > max)
                  out.append(", ...");

               break;
               }
            }

         out.append(" }\n");
         return;
         }

      // metaclass object?
      //
      if (className.equals("java.lang.Class"))
         {
         out.append("Class(" + value.typeName() + ")\n");
         return;
         }

      // regular object
      //
      
      if (className.startsWith("java.lang."))
         { // shorten well known type names
         className = className.substring("java.lang.".length());
         }

      if (className.equals("Boolean")  ||
          className.equals("Byte")     ||
          className.equals("Short")    ||
          className.equals("Integer")  ||
          className.equals("Long")     ||
          className.equals("Float")    ||
          className.equals("Double")   )
         { // standard type
         out.append(className + "(" + value + ")\n");
         return;
         }

      if (className.equals("Character"))
         { // standard type
         out.append(className + "(" + formatCharValue(value) + ")\n");
         return;
         }

      if (className.equals("String"))
         { // standard type

      // out.append(className + "(" + value + ")\n");
         
         // we format the string ourself, character by character, in order to handle non-text characters (newlines)
         // and excessively long strings (text buffers, etc.)
         //

         RemoteArray ary = (RemoteArray)((RemoteObject)value).getFieldValue("value");
         int         off = ((RemoteInt)((RemoteObject)value).getFieldValue("offset")).get();
         int         cnt = ((RemoteInt)((RemoteObject)value).getFieldValue("count")).get();
         int         siz = ary.getSize();
         int         max = cnt > MAXSTR ? MAXSTR : cnt;

         // range check values just in case string's state is faulty
         //
         if (off < 0 || cnt < 0 || off + cnt > siz)
            {
            out.append(className + "(<badstring>)\n");
            return;
            }

         out.append(className + "(");
         if (max != 0)
            {
            RemoteValue[] elts = ary.getElements(off, off + max - 1);
            for (int i = 0;;)
               {
               char c = elts[i].toString().charAt(0);

               if (c < 0x20 || c > 0x7f) out.append("[0x" + Integer.toHexString((int)c) + "]");
               else                      out.append(c);

               if (++i < max)
                  continue;

               if (cnt > max)
                  out.append("...");

               break;
               }
            }
         out.append(")\n");
         return;
         }

      if (className.equals("java.util.HashTable"))
         { // watch out: this class's toString method prints the entire hashtable!
         out.append(className + "(...)\n");
         return;
         }

      String selfDescribe = value.toString();
      
      if (selfDescribe.indexOf('@', 0) > 0)
         { // default description is not very interesting
         out.append(className + "\n");
         }
      else
         { // custom description might yield something more useful
         out.append(selfDescribe.replace('\n', ' ') + "\n");
         }
      }

   static String
   formatCharValue(RemoteValue charValue)
      {
      char c = charValue.toString().charAt(0);

      if (c < 0x20 || c > 0x7f)
         return "0x" + Integer.toHexString((int)c);
      else
         return "'" + c + "'";
      }
   }
