// A panel split like this:
//
//         +------------------+
//         |                  |
//         |------------------|
//         |                  |
//         +------------------+
//
// 27 Mar 1996 Derek Lieber
//
package ui;
import java.awt.*;

public class HorizontallySplitPanel extends SplitPanel
   {
   int componentY[];  // component positions
   int componentH[];  // component heights
   int topTarget;     // index of component above "hot" splitter
   int bottomTarget;  // index of component below "hot" splitter
   int initialY;      // cursor position when mouse first pressed
   int currentY;      // cursor position as mouse is dragged

   public
   HorizontallySplitPanel()
      {
   // setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
      }
      
   // Called by AWT.
   //
   synchronized
   public Dimension
   minimumLayoutSize(Container me) //- implements LayoutManager
      {
      // this method never seems to get called
      //
      System.out.println("HorizontallySplitPanel.minimumLayoutSize: unexpected call");
      
      Dimension min = new Dimension(0,0);
      int       n   = getComponentCount();
      for (int i = 0; i < n; ++i)
         {
         Dimension d = getComponent(i).getMinimumSize();
         if (d.width > min.width)
            min.width = d.width;
         min.height += d.height + splitterThickness;
         }
      min.height -= splitterThickness;
      return min;
      }

   // Called by AWT.
   //
   synchronized
   public Dimension
   getPreferredSize() //- overrides Component
      {
      int w = 0;
      int h = 0;
      int n = getComponentCount();

      for (int i = 0; i < n; ++i)
         {
         Dimension d = getComponent(i).getPreferredSize();
         w += d.width;
         h += d.height + splitterThickness;
         }
      
      h -= splitterThickness; // for "n" components there are "n-1" splitters
      return new Dimension(w, h);
      }

   // Called by AWT.
   //
   synchronized
   public void
   doLayout() //- overrides Container
      {
      // normalize layout constraints
      if (numComponents != getComponentCount())
         {
         numComponents = getComponentCount();
         normalize();
         }

      if (numComponents == 0)
         return;

      panelW = getSize().width;
      panelH = getSize().height;
      
      componentY = new int[numComponents];
      componentH = new int[numComponents];
      
      // divvy up available space
      int space = panelH - (numComponents - 1) * splitterThickness;
      for (int i = 0; i < numComponents; ++i)
         componentH[i] = (int)(space * weights[i]);
      
      // correct for floating point roundoff
      int error = space;
      for (int i = 0; i < numComponents; ++i)
         error -= componentH[i];
      componentH[numComponents - 1] += error;

      // assign positions
      for (int i = 0, y = 0; i < numComponents; ++i)
         {
         componentY[i] = y;
         y += componentH[i] + splitterThickness;
         }

      // adjust screen appearance
      for (int i = 0; i < numComponents; ++i)
         {
         Component c = getComponent(i);
         c.setBounds(0, componentY[i], panelW, componentH[i]);
         c.validate();
         }
      }

   // Called by AWT.
   //
   synchronized
   public void
   paint(Graphics g) //- overrides Component
      {
      super.paint(g);
      for (int i = 0; i < numComponents; ++i)
         {
         int y = componentY[i] + componentH[i] + splitterThickness / 2;

         g.setColor(getBackground()); g.drawLine(0, y-2, panelW, y-2);
         g.setColor(Color.gray);      g.drawLine(0, y-1, panelW, y-1);
         g.setColor(Color.black);     g.drawLine(0, y  , panelW, y  );
         g.setColor(Color.white);     g.drawLine(0, y+1, panelW, y+1);
         g.setColor(Color.lightGray); g.drawLine(0, y+2, panelW, y+2);
         }
      }
      
   // Called by SplitPanel.mouseDown().
   //
   synchronized
   void
   identifyTargets(int x, int y) //- implements SplitPanel
      {
      // identify components that straddle current cursor position
      for (int i = 1; i < numComponents; ++i)
         if (componentY[i] > y)
            {
            topTarget    = i-1;
            bottomTarget = i;
            initialY = currentY = componentY[i-1] + componentH[i-1] + splitterThickness / 2;
            break;
            }
      }
      
   // Called by SplitPanel.mouseDown/Drag().
   //
   synchronized
   void
   drawSplitter(int x, int y) //- implements SplitPanel
      {
      currentY = y;
      
      dragG.setColor(getBackground());
      dragG.fillRect(0, 0, panelW, panelH);

      int minY = 0;
      if (topTarget > 0)
         minY = componentY[topTarget];
      minY += minThickness;

      int maxY = panelH - 1;
      if (bottomTarget < numComponents - 1)
         maxY = componentY[bottomTarget + 1];
      maxY -= minThickness;

      // don't under-run
      if (currentY < minY)
         currentY = minY;
      
      // don't over-run
      if (currentY > maxY)
          currentY = maxY;

      for (int i = 0; i < numComponents; ++i)
         {
         if (i == topTarget)
            y = currentY;
         else
            y = componentY[i] + componentH[i] + splitterThickness / 2;

         dragG.setColor(getBackground()); dragG.drawLine(0, y-2, panelW, y-2);
         dragG.setColor(Color.gray);      dragG.drawLine(0, y-1, panelW, y-1);
         dragG.setColor(Color.black);     dragG.drawLine(0, y  , panelW, y  );
         dragG.setColor(Color.white);     dragG.drawLine(0, y+1, panelW, y+1);
         dragG.setColor(Color.lightGray); dragG.drawLine(0, y+2, panelW, y+2);
         }
      }

   // Called by SplitPanel.mouseUp().
   //
   synchronized
   void
   resplit() //- implements SplitPanel
      {
      // adjust positions and sizes of affected components
      int deltaY = currentY - initialY;
      componentH[topTarget]    += deltaY;
      componentY[bottomTarget] += deltaY;
      componentH[bottomTarget] -= deltaY;

      // recompute space allocations
      int space = panelH - (numComponents - 1) * splitterThickness;
      for (int i = 0; i < numComponents; ++i)
         weights[i] = (double)componentH[i] / (double)space;
      
      // adjust screen appearance
      for (int i = topTarget; i <= bottomTarget; ++i)
         {
         Component c = getComponent(i);
         c.setBounds(0, componentY[i], panelW, componentH[i]);
         c.validate();
         }
      }
   }
