// A multi-purpose button.
// Its face can be rendered from either an Image or a String.
// Its behavior can be either "regular", "sticky", or "autorepeat".
// It can be blinked.
// 18 Sep 1997 Derek Lieber.
//
package ui;
import  java.awt.*;
import  java.awt.event.*;

public class MultiButton extends FastComponent
   implements Constants, MouseListener, AlarmListener
   {
   private Object                normalVisual;   // Image or String to be rendered on button face, during ON portion of blink cycle (null --> nothing to render)
   private Object                alternateVisual;// Image or String to be rendered during OFF portion of blink cycle (null --> nothing to render)
   private Object                currentVisual;  // current blink state (== normalVisual or alternateVisual)
   private String                description;    // descriptive text for "flyover help" (null --> no flyover help)
   private boolean               isSticky;       // is this a sticky button (stays "selected" once pressed)?
   private boolean               isAutoRepeater; // is this an autorepeat button (delivers "click" events while pressed)?
   private boolean               isActivated;    // is sticky button in "activated" state?
   private boolean               isPressed;      // is this button currently "pressed in"?
   private ActionListener        actionListener; // listeners on this button
   private static AlarmGenerator blinkGenerator; // blink event generator
   private static AlarmGenerator repeatGenerator;// autorepeat event generator

   //-----------//
   // interface //
   //-----------//

   public static final int     IS_REGULAR      = 0; // kinds of buttons
   public static final int     IS_STICKY       = 1; // ...that can
   public static final int     IS_AUTOREPEATER = 2; // ......be created
   
   // Create regular button.
   // Taken: image or string to be rendered on face (null --> blank face)
   //        text for flyover help                  (null --> no help)
   //
   public 
   MultiButton(Object visual, String description)
      {
      this(visual, description, IS_REGULAR);
      }

   // Create regular, sticky, or autorepeat button.
   // Taken: image or string to be rendered on face (null --> blank face)
   //        text for flyover help                  (null --> no help)
   //        kind of button to create               (one of IS_XXX, above)
   //
   public 
   MultiButton(Object visual, String description, int kind)
      {
      normalVisual = currentVisual = visual;
      this.description    = description;
      this.isSticky       = kind == IS_STICKY;
      this.isAutoRepeater = kind == IS_AUTOREPEATER;
      addMouseListener(this);
      if (repeatGenerator == null)
         repeatGenerator = new AlarmGenerator(true);
      }

   // Listen for mouse clicks on this button.
   //
   synchronized
   public void 
   addActionListener(ActionListener listener)
      {
      actionListener = AWTEventMulticaster.add(actionListener, listener);
      }

   // Set/reset this (sticky) button's "activated" state.
   //
   synchronized
   public void 
   setActivated(boolean b)
      {
      isActivated = b;
      isPressed   = b;
      Util.redraw(this);
      }

   // Allow/prohibit mouse clicking on this button.
   //
   synchronized
   public void 
   setEnabled(boolean b) //- extends Component
      {
      super.setEnabled(b);
   // Util.redraw(this);
      }

   // Get flyover help text.
   //
   synchronized
   public String
   getDescription()
      {
      return description;
      }

   // Start blinking.
   // Taken: Image or String to display during OFF portion of blink cycle.
   //
   public void
   startBlinking(Object alternateVisual)
      {
      if (blinkGenerator == null)
         blinkGenerator = new AlarmGenerator(false);
      this.alternateVisual = alternateVisual;
      blinkGenerator.on(this);
      }
   
   // Stop blinking.
   //
   public void
   stopBlinking()
      {
      if (blinkGenerator == null)
         return;
      blinkGenerator.off();
      if (currentVisual != normalVisual)
         {
         currentVisual = normalVisual;
         Util.redraw(this);
         }
      }

   //------------------------//
   // implementation details //
   //------------------------//
   
   public void
   alarmFired(AlarmGenerator source) //- implements AlarmListener
      {
      if (source == repeatGenerator)
         {
         deliverClick("autorepeat");
         }
      else
         {
         if (currentVisual == alternateVisual) currentVisual = normalVisual;
         else                                  currentVisual = alternateVisual;
         Util.redraw(this);
         }
      }

   synchronized
   public void 
   mousePressed(MouseEvent e) //- implements MouseListener
      {
      if (isActivated)
         return;
      
      isPressed = true;
      Util.redraw(this);
      
      if (!isAutoRepeater)
         return;

      deliverClick("click"); // for autorepeater, deliver click on "press" not "release"
      
      repeatGenerator.on(this);
      }
      
   synchronized
   public void 
   mouseReleased(MouseEvent e) //- implements MouseListener
      { 
      if (isActivated)
         return;

      if (isSticky)
         {
         isActivated = true;
         deliverClick("click");
         }
      else if (isAutoRepeater)
         {
         isPressed = false;
         repeatGenerator.off();
         }
      else
         {
         isPressed = false;
         deliverClick("click");
         }
         
      Util.redraw(this);
      }

   synchronized
   public void 
   mouseEntered(MouseEvent e) //- implements MouseListener
      {
      }
      
   synchronized
   public void 
   mouseExited(MouseEvent e) //- implements MouseListener
      {
      if (!isPressed || isActivated)
         return;

      isPressed   = false;
      isActivated = false;
      if (isAutoRepeater)
         repeatGenerator.off();

      Util.redraw(this);
      }
      
   public void 
   mouseClicked(MouseEvent e) //- implements MouseListener
      {
      }

   synchronized
   void
   deliverClick(String kind)
      {
      if (actionListener != null)
         {
         ActionEvent e = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, kind);
         actionListener.actionPerformed(e);
         }
      }

   // See also: MultiLabel
   //
   synchronized
   public Dimension
   getPreferredSize() //- overrides Component
      {
      int w, h;
      
      if (currentVisual == null)
         {
         w = 0;
         h = 0;
         }
      else if (currentVisual instanceof Image)
         {
         Image image = (Image)currentVisual;
         w = image.getWidth(this);
         h = image.getHeight(this);
         }
      else if (currentVisual instanceof String)
         {
         FontMetrics metrics = getFontMetrics(getFont());
         w = metrics.stringWidth((String)currentVisual);
         h = metrics.getAscent() + metrics.getMaxDescent();
         }
      else
         {
         w = 0;
         h = 0;
         }

      return new Dimension(BUTTON_BORDER_THICKNESS + w + BUTTON_BORDER_THICKNESS,
                           BUTTON_BORDER_THICKNESS + h + BUTTON_BORDER_THICKNESS);
      }
      
   synchronized
   public Dimension
   getMinimumSize() //- overrides Component
      {
      return getPreferredSize();
      }
      
   synchronized
   public Dimension
   getMaximumSize() //- overrides Component
      {
      return getPreferredSize();
      }
      
   // See also: MultiLabel
   //
   synchronized
   public void 
   paint(Graphics g) //- overrides Component
      {
      Rectangle r = Util.getLocalBounds(this);

      if (isPressed) Util.drawInsetBorder(g, r);
      else           Util.drawOutsetBorder(g, r);

      int x = r.x          + BUTTON_BORDER_THICKNESS;
      int y = r.y          + BUTTON_BORDER_THICKNESS;
      int w = r.width  - 2 * BUTTON_BORDER_THICKNESS;
      int h = r.height - 2 * BUTTON_BORDER_THICKNESS;
      
   // if (isEnabled() == false)
   //    {
   //    g.setColor(SystemColor.inactiveCaptionBorder);
   //    g.fillRect(x, y, w, h);
   //    return;
   //    }
         
      g.setColor(isActivated ? SystemColor.controlLtHighlight : SystemColor.controlHighlight);
      g.fillRect(x, y, w, h);
      
      if (isPressed)
         {
         x += 1;
         y += 1;
         w -= 1;
         h -= 1;
         }
         
      if (currentVisual == null)
         {
         }
      else if (currentVisual instanceof Image)
         {
         g.drawImage((Image)currentVisual, x, y, w, h, null);
         }
      else if (currentVisual instanceof String)
         {
         String      text    = (String)currentVisual;
         Font        font    = getFont();
         FontMetrics metrics = getFontMetrics(font);
         int         fontH   = metrics.getAscent() + metrics.getMaxDescent();
         int         fontW   = metrics.stringWidth(text);

         x += (w - fontW) / 2;
         y += (h - fontH) / 2 + metrics.getAscent();

         g.setFont(font);
         g.setColor(getForeground());
         g.drawString(text, x, y);
         }
      }
   }
