// A multi-purpose label.
// Its face can be rendered from either an Image or a String.
// It can be blinked.
// 18 Sep 1997 Derek Lieber.
//
package ui;
import java.awt.*;
import java.awt.event.*;

public class MultiLabel extends FastComponent
   implements AlarmListener
   {
   private static AlarmGenerator blinkGenerator;  // blink event generator
   private Object                normalVisual;    // Image or String to be rendered during ON  portion of blink cycle (null --> nothing to render)
   private Object                alternateVisual; // Image or String to be rendered during OFF portion of blink cycle (null --> nothing to render)
   private Object                currentVisual;   // current blink state (== normalVisual or alternateVisual)
           Insets                insets;          // space to leave around edges

   //-----------//
   // interface //
   //-----------//
   
   // Create.
   // Taken: image or string to be rendered on face (null --> blank face)
   //
   public 
   MultiLabel(Object visual)
      {
      this(visual, new Insets(0,0,0,0));
      }

   // Create.
   // Taken: image or string to be rendered on face (null --> blank face)
   //        space to leave around edges
   //
   public 
   MultiLabel(Object visual, Insets insets)
      {
      this.currentVisual = this.normalVisual = visual;
      this.insets = insets;
///   setBackground(SystemColor.controlHighlight);
      }

   public void
   setVisual(Object visual)
      {
      this.currentVisual = this.normalVisual = visual;

      // layout our container to accomodate possible size change
      //
      invalidate();
      Util.redraw(getParent());
      }

   public Object
   getVisual()
      {
      return normalVisual;
      }

   // Start blinking.
   // Taken: Image or String to display during OFF portion of blink cycle.
   //
   public void
   startBlinking(Object alternateVisual)
      {
      if (blinkGenerator == null)
         blinkGenerator = new AlarmGenerator(false);
      this.alternateVisual = alternateVisual;
      blinkGenerator.on(this);
      }
   
   // Stop blinking.
   //
   public void
   stopBlinking()
      {
      if (blinkGenerator == null)
         return;
      blinkGenerator.off();
      if (currentVisual != normalVisual)
         {
         currentVisual = normalVisual;
         Util.redraw(this);
         }
      }

   //------------------------//
   // implementation details //
   //------------------------//
   
   public void
   alarmFired(AlarmGenerator source) //- implements AlarmListener
      {
      if (currentVisual == alternateVisual) currentVisual = normalVisual;
      else                                  currentVisual = alternateVisual;
      Util.redraw(this);
      }

   // See also: MultiButton
   //
   synchronized
   public Dimension
   getPreferredSize() //- overrides Component
      {
      int w, h;
      
      if (currentVisual == null)
         {
         w = 0;
         h = 0;
         }
      else if (currentVisual instanceof Image)
         {
         Image image = (Image)currentVisual;
         w = image.getWidth(this);
         h = image.getHeight(this);
         }
      else if (currentVisual instanceof String)
         {
         FontMetrics metrics = getFontMetrics(getFont());
         w = metrics.stringWidth((String)currentVisual);
         h = metrics.getAscent() + metrics.getMaxDescent();
         }
      else
         {
         w = 0;
         h = 0;
         }

      return new Dimension(insets.left + w + insets.right, insets.top + h + insets.bottom);
      }
      
   synchronized
   public Dimension
   getMinimumSize() //- overrides Component
      {
      return getPreferredSize();
      }
      
   synchronized
   public Dimension
   getMaximumSize() //- overrides Component
      {
      return getPreferredSize();
      }
      
   // See also: MultiButton
   //
   synchronized
   public void 
   paint(Graphics g) //- overrides Component
      {
      Rectangle r = Util.getLocalBounds(this);

      int x = r.x      + insets.left;
      int y = r.y      + insets.top;
      int w = r.width  - insets.left - insets.right;
      int h = r.height - insets.top  - insets.bottom;
      
      g.setColor(getBackground());
      g.fillRect(x, y, w, h);
      
      if (currentVisual == null)
         {
         }
      else if (currentVisual instanceof Image)
         {
         g.drawImage((Image)currentVisual, x, y, w, h, null);
         }
      else if (currentVisual instanceof String)
         {
         String      text    = (String)currentVisual;
         Font        font    = getFont();
         FontMetrics metrics = getFontMetrics(font);
         int         fontH   = metrics.getAscent() + metrics.getMaxDescent();
         int         fontW   = metrics.stringWidth(text);

         x += (w - fontW) / 2;
         y += (h - fontH) / 2 + metrics.getAscent();

         g.setFont(font);
         g.setColor(getForeground());
         g.drawString(text, x, y);
         }
      }
   }
