// Arrange a panel into rows or columns of subcomponents that can be resized individually
// by dragging on "splitter" bars. When a subcomponent is added to the panel, its vertical
// or horizontal weight is specified as a layout constraint and will be used to set the 
// panel's height or width.
//
// For example, the following creates a panel containing three text areas:
//
//    SplitPanel panel  = new VerticallySplitPanel();
//    panel.add(new TextArea(), "25");
//    panel.add(new TextArea(), "25");
//    panel.add(new TextArea(), "50");
//
// On the screen, the panel looks something like this:
//
//    +----------+----------+---------------------+
//    |          |          |                     |
//    |   25%    |    25%   |        50%          |
//    |          |          |                     |
//    +----------+----------+---------------------+
//
// 27 Mar 1996 Derek Lieber
//
//!!TODO: consolidate common painting code
//!!TODO: repaint only the "hot" splitter when dragging (don't repaint other splitters)
//
package ui;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public abstract class SplitPanel extends FastPanel
   implements LayoutManager, MouseListener, MouseMotionListener
   {
   Vector     constraints;            // layout constraints specified by addLayoutComponent()

   int        numComponents;          // number of components to manage
   double     weights[];              // how to apportion space among them (normalized to values whose sum is 1.0)

   int        panelW;                 // panel dimensions
   int        panelH;
   final int  splitterThickness = 5;  // space between components
   final int  minThickness      = 5;  // minimum size of a component when resplitting

   boolean    dragging;               // splitter animation in progress?
   Graphics   dragG;                  // graphics context for splitter animation

   public
   SplitPanel()
      {
      constraints = new Vector();
      setLayout(this);
      addMouseListener(this);
      addMouseMotionListener(this);
      }

   synchronized
   public void
   addLayoutComponent(String weight, Component component) //- implements LayoutManager
      {
      constraints.addElement(Double.valueOf(weight));
      }

   synchronized
   public void
   removeLayoutComponent(Component component) //- implements LayoutManager
      {
      for (int i = 0, n = getComponentCount(); i < n; ++i)
         if (getComponent(i) == component)
            {
            constraints.removeElementAt(i);
            numComponents -= 1;
            return;
            }
      }

   synchronized
   public Dimension
   preferredLayoutSize(Container me) //- implements LayoutManager
      {
      // this method never seems to get called
      //
      System.out.println("SplitPanel.preferredLayoutSize: unexpected call");
      return getPreferredSize();
      }

   synchronized
   public void
   layoutContainer(Container me) //- implements LayoutManager
      {
      // this method never seems to get called
      //
      System.out.println("SplitPanel.layoutContainer: unexpected call");
      }

   public void mouseClicked( MouseEvent e) {} //- implements MouseListener
   public void mouseEntered( MouseEvent e) {} //- implements MouseListener
   public void mouseExited(  MouseEvent e) {} //- implements MouseListener
   public void mouseMoved(   MouseEvent e) {} //- implements MouseMotionListener
   
   synchronized
   public void 
   mousePressed(MouseEvent e) //- implements MouseListener
      { 
      int x = e.getX();
      int y = e.getY();

      for (int i = 0; i < numComponents; ++i)
         getComponent(i).setVisible(false);

      dragging = true;
      dragG    = getGraphics();
      identifyTargets(x, y);
      drawSplitter(x, y);
      }
   
   synchronized
   public void 
   mouseDragged( MouseEvent e) //- implements MouseMotionListener
      {
      if (dragging)
         {
         int x = e.getX();
         int y = e.getY();
         int w = getSize().width;
         int h = getSize().height;

         // beware: mouse can move out of bounds
         if ( x <  0 ) x = 0;
         if ( x >= w ) x = w - 1;
         if ( y <  0 ) y = 0;
         if ( y >= h ) y = h - 1;
            
         drawSplitter(x, y);
         }
      }

   synchronized
   public void 
   mouseReleased(MouseEvent e) //- implements MouseListener
      { 
      if (dragging)
         {
         dragging = false;
         dragG.dispose();
         
         repaint();
         resplit();
        
         for (int i = 0; i < numComponents; ++i)
            getComponent(i).setVisible(true);
         }
      }

   // Subclass responsibility.
   //
   public abstract Dimension minimumLayoutSize(Container c);
   abstract void             identifyTargets(int x, int y);
   abstract void             drawSplitter(int x, int y);
   abstract void             resplit();

   // Normalize layout constraints.
   // Taken:    constraints[]
   // Returned: weights[]
   //
   synchronized
   void
   normalize()
      {
      if (constraints.size() != numComponents)
         { // missing constraints 

         // Some components were probably added to container with "Container.add(Component)" rather
         // than "Container.add(Component, String)". Ignore the constraints and apportion space equally.

      // System.err.println("SplitPanel: missing layout constraints (using default layout)");
         constraints = new Vector();
         for (int i = 0; i < numComponents; ++i)
            constraints.addElement(new Double(1.0 / numComponents));
         }

      // tally
      double sum = 0;
      weights = new double[numComponents];
      for (int i = 0; i < numComponents; ++i)
         sum += weights[i] = ((Double)constraints.elementAt(i)).doubleValue();

      // normalize
      for (int i = 0; i < numComponents; ++i)
         weights[i] /= sum;
      }
   }
