// A panel split like this:
//
//         +-----------------+
//         |        |        |
//         |        |        |
//         |        |        |
//         +-----------------+
//
// 27 Mar 1996 Derek Lieber
//
package ui;
import java.awt.*;

public class VerticallySplitPanel extends SplitPanel
   {
   int componentX[];  // component positions
   int componentW[];  // component widths
   int leftTarget;    // index of component to left of "hot" splitter
   int rightTarget;   // index of component to right of "hot" splitter
   int initialX;      // cursor position when mouse first pressed
   int currentX;      // cursor position as mouse is dragged

   public
   VerticallySplitPanel()
      {
   // setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
      }

   // Called by AWT.
   //
   synchronized
   public Dimension
   minimumLayoutSize(Container me) //- implements LayoutManager.
      {
      // this method never seems to get called
      //
      System.out.println("VerticallySplitPanel.minimumLayoutSize: unexpected call");

      Dimension min = new Dimension(0,0);
      int       n   = getComponentCount();
      for (int i = 0; i < n; ++i)
         {
         Dimension d = getComponent(i).getMinimumSize();
         min.width  += d.width + splitterThickness;
         if (d.height > min.height)
            min.height = d.height;
         }
      min.width -= splitterThickness;
      return min;
      }

   // Called by AWT.
   //
   synchronized
   public Dimension
   getPreferredSize() //- overrides Component
      {
      int w = 0;
      int h = 0;
      int n = getComponentCount();

      for (int i = 0; i < n; ++i)
         {
         Dimension d = getComponent(i).getPreferredSize();
         w += d.width + splitterThickness;
         h += d.height;
         }

      w -= splitterThickness; // for "n" components there are "n-1" splitters
      return new Dimension(w, h);
      }

   // Called by AWT.
   //
   synchronized
   public void
   doLayout() //- overrides Container
      {
      // normalize layout constraints
      if (numComponents != getComponentCount())
         {
         numComponents = getComponentCount();
         normalize();
         }
   
      if (numComponents == 0)
         return;

      panelW = getSize().width;
      panelH = getSize().height;
      
      componentX = new int[numComponents];
      componentW = new int[numComponents];
      
      // divvy up available space
      int space = panelW - (numComponents - 1) * splitterThickness;
      for (int i = 0; i < numComponents; ++i)
         componentW[i] = (int)(space * weights[i]);
      
      // correct for floating point roundoff
      int error = space;
      for (int i = 0; i < numComponents; ++i)
         error -= componentW[i];
      componentW[numComponents - 1] += error;

      // assign positions
      for (int i = 0, x = 0; i < numComponents; ++i)
         {
         componentX[i] = x;
         x += componentW[i] + splitterThickness;
         }

      // adjust screen appearance
      for (int i = 0; i < numComponents; ++i)
         {
         Component c = getComponent(i);
         c.setBounds(componentX[i], 0, componentW[i], panelH);
         c.validate();
         }
      }

   // Called by AWT.
   //
   synchronized
   public void
   paint(Graphics g) //- overrides Component
      {
      super.paint(g);
      for (int i = 0; i < numComponents; ++i)
         {
         int x = componentX[i] + componentW[i] + splitterThickness / 2;
         
         g.setColor(getBackground()); g.drawLine(x-2, 0, x-2, panelH);
         g.setColor(Color.gray);      g.drawLine(x-1, 0, x-1, panelH);
         g.setColor(Color.black);     g.drawLine(x  , 0, x  , panelH);
         g.setColor(Color.white);     g.drawLine(x+1, 0, x+1, panelH);
         g.setColor(Color.lightGray); g.drawLine(x+2, 0, x+2, panelH);
         }
      }
      
   // Called by SplitPanel.mouseDown().
   //
   synchronized
   void
   identifyTargets(int x, int y) //- implements SplitPanel
      {
      // identify components that straddle current cursor position
      for (int i = 1; i < numComponents; ++i)
         if (componentX[i] > x)
            {
            leftTarget  = i-1;
            rightTarget = i;
            initialX = currentX = componentX[i-1] + componentW[i-1] + splitterThickness / 2;
            break;
            }
      }
      
   // Called by SplitPanel.mouseDown/Drag().
   //
   synchronized
   void
   drawSplitter(int x, int y) //- implements SplitPanel
      {
      currentX = x;
      
      dragG.setColor(getBackground());
      dragG.fillRect(0, 0, panelW, panelH);

      int minX = 0;
      if (leftTarget > 0)
         minX = componentX[leftTarget];
      minX += minThickness;

      int maxX = panelW - 1;
      if (rightTarget < numComponents - 1)
         maxX = componentX[rightTarget + 1];
      maxX -= minThickness;

      // don't under-run
      if (currentX < minX)
         currentX = minX;
      
      // don't over-run
      if (currentX > maxX)
          currentX = maxX;

      for (int i = 0; i < numComponents; ++i)
         {
         if (i == leftTarget)
            x = currentX;
         else
            x = componentX[i] + componentW[i] + splitterThickness / 2;
         
         dragG.setColor(getBackground()); dragG.drawLine(x-2, 0, x-2, panelH);
         dragG.setColor(Color.gray);      dragG.drawLine(x-1, 0, x-1, panelH);
         dragG.setColor(Color.black);     dragG.drawLine(x  , 0, x  , panelH);
         dragG.setColor(Color.white);     dragG.drawLine(x+1, 0, x+1, panelH);
         dragG.setColor(Color.lightGray); dragG.drawLine(x+2, 0, x+2, panelH);
         }
      }
   
   // Called by SplitPanel.mouseUp().
   //
   synchronized
   void
   resplit() //- implements SplitPanel
      {
      // adjust positions and sizes of affected components
      int deltaX = currentX - initialX;
      componentW[leftTarget]  += deltaX;
      componentX[rightTarget] += deltaX;
      componentW[rightTarget] -= deltaX;

      // recompute space allocations
      int space = panelW - (numComponents - 1) * splitterThickness;
      for (int i = 0; i < numComponents; ++i)
         weights[i] = (double)componentW[i] / (double)space;
      
      // adjust screen appearance
      for (int i = leftTarget; i <= rightTarget; ++i)
         {
         Component c = getComponent(i);
         c.setBounds(componentX[i], 0, componentW[i], panelH);
         c.validate();
         }
      }
   }
